// implementace obecneho (ne)orientovaneho (ne)ohodnoceneho grafu

#define DEBUG 0

#include "graph.h"

#if DEBUG
#include <iostream>
using namespace std;
#endif


//////////////////////////////////////////////////////////////////////////////

// elem_t

elem_t::elem_t(int Value): value(Value) {}

inline void elem_t::setvalue(int Value) { value=Value; }

inline int elem_t::getvalue() { return value; }

//////////////////////////////////////////////////////////////////////////////

// edge_t

edge_t::edge_t(node_t *From, node_t *To, int Value)
: elem_t(Value)
{
   from=From;
   to=To;
   from->edges.insert(this);
   to->edges.insert(this);
#if DEBUG
   cerr << "vytvarim hranu s hodnotou "<< getvalue() << "\n";
#endif
}

edge_t::~edge_t() {  // destruktor - rusi reference v uzlech
   from->edges.erase(this);
   to->edges.erase(this);
#if DEBUG
   cerr << "rusim hranu s hodnotou "<< getvalue() << "\n";
#endif
}

node_t *edge_t::isfrom(node_t *From) {  // test vystupni orientovane
   if(!From) return from;
   return (from==From)? to : NULL;
}

bool edge_t::isfromto(node_t *From, node_t *To) {  // test orient.
   return (from==From && to==To);
}


nedge_t::nedge_t(node_t *From, node_t *To, int Value)
: edge_t(From, To, Value) {}

node_t *nedge_t::isfrom(node_t *From) {  // test neorientovane
   if (!From || to==From)  return from;  // NULL a opacny smer
   if (from==From) return to;
   return NULL;
}

bool nedge_t::isfromto(node_t *From, node_t *To) {  // test neorient.
   return (from==From && to==To || from==To && to==From);
}


//////////////////////////////////////////////////////////////////////////////

// node_t

node_t::node_t(int Value): elem_t(Value)
{
#if DEBUG
   cerr << "vytvarim uzel s hodnotou " << getvalue() << "\n";
#endif

}

edgelst_t node_t::getedges() { return edges; }

node_t::~node_t() {  // rusi hrany
   while (edges.begin()!=edges.end()) delete (*edges.begin());
#if DEBUG
   cerr << "rusim uzel s hodnotou " << getvalue() << "\n";
#endif
}

//////////////////////////////////////////////////////////////////////////////
// graph_t

bool graph_t::isedge(node_t *From, node_t *To) {
   bool aux=false;
   for (edgelst_t::iterator i=edges.begin();
        i!=edges.end() && !(aux=(*i)->isfromto(From, To)); ++i);
   return aux;
}

bool graph_t::erase(node_t* Node, bool Edges) {
   bool aux=false;
   nodelst_t::iterator n;
   if ((n=nodes.find(Node))!=nodes.end()) {
      edgelst_t nedges = Node->getedges();  // ziska seznam hran uzlu
      if((aux = Edges || nedges.empty())!=0) {
         for(edgelst_t::iterator i=nedges.begin(); i!=nedges.end(); ++i)
            edges.erase(*i);          // maze referece techto hran v grafu
         delete Node;                 // nakonec fyzicky uzlel a hrany
         nodes.erase(n);              // maze referenci na uzel v grafu
      }
   }
   return aux;
}

void graph_t::erase(edge_t* Edge) {
   edgelst_t::iterator aux;
   if ((aux=edges.find(Edge))!=edges.end()) {
      delete Edge;
      edges.erase(aux);
   }
}

node_t *graph_t::insert_node(int Value) {
   node_t *node = new node_t(Value);
   nodes.insert(node);
   return node;
}

edge_t *graph_t::insert_edge(node_t *From, node_t *To, bool Unique, int Value)

{
   edge_t *edge=NULL;
   if(nodes.find(From)!=nodes.end() && nodes.find(To)!=nodes.end() &&
      !(Unique && isedge(From, To)) ) {
   // vlozi (ne)unikatni orientovanou hranu, pokud oba uzly patri do grafu
      edge = new edge_t(From, To, Value);
      edges.insert(edge);
   }
   return edge;
}

void graph_t::clear() {  // fyzicky rusi uzly (a automaticky i hrany) grafu
   for(nodelst_t::iterator i=nodes.begin(); i!=nodes.end(); ++i)
      delete (*i);
   nodes.clear();
   edges.clear();
}

graph_t::~graph_t() { clear(); }


edge_t *ngraph_t::insert_edge(node_t *From, node_t *To, bool Unique, int
 Value)
{
   nedge_t *edge=NULL;
   if(nodes.find(From)!=nodes.end() && nodes.find(To)!=nodes.end() &&
      !(Unique && isedge(From, To)) ) {
   // vlozi (ne)unikatni orientovanou hranu, pokud oba uzly patri do grafu
      edge = new nedge_t(From, To, Value);
      edges.insert(edge);
   }
   return edge;
}

//////////////////////////////////////////////////////////////////////////////


#if DEBUG
int main() {                    // kontrolni programek (na dedicnosti, ...)
   node_t  *pn1, *pn2, *pn3, *pn4;
   edge_t  *pe1, *pe2;
   graph_t  og;
   ngraph_t ng;
   pn1 = og.insert_node(1);          // uzel s hodnotou
   pn2 = og.insert_node(2);
   pn3 = ng.insert_node(3);
   pn4 = ng.insert_node(4);
   if(og.insert_edge(pn1, pn3, true))
      cerr<<"Chyba1\n";                    // nesmi vypsat (hrana mezi grafy)
cerr<<"0\n";
   pe1 = og.insert_edge(pn1, pn2, true, 1);  // orientovana hrana s hodnotou
   if(pe1->isfrom(pn2)) cerr<<"Chyba2\n";  // nesmi vypsat (spatny smer)

   if(og.insert_edge(pn1, pn2, true, 2))
      cerr<<"Chyba3\n";                     // nesmi vypsat (unikatni hrana)

   if(!(pe2 = og.insert_edge(pn2, pn1, true, 3))) // opacna hrana
      cerr<<"Chyba4\n";                    // nesmi vypsat
cerr<<"1\n";
   og.erase(pn2);                      // rusi uzel a hrany
cerr<<"2\n";
   if(!(pe2=ng.insert_edge(pn3, pn4, false, 4)))  // neor. hrana s hodnotou
      cerr<<"Chyba5\n";                    // neorient. hrana)

   if (ng.insert_edge(pn4, pn3, true, 5))
      cerr<<"Chyba6\n";                    // nesmi vypsat (opacna hrana)
cerr<<"3\n";

   if(pe2->isfrom(pn4)!=pn3) cerr<<"Chyba7\n";  // nesmi vypsat (ok)

}
#endif




