// cesty resp. jen Boruvkova

#define DEBUG 1

#include "path.h"

#if DEBUG
#include <iostream>
using namespace std;
#endif

// pomocna pro hledani minima
node_t *min_value_remove(nodelst_t &nodes) {  // vyjme uzel s mnimalni cenou
   nodelst_t::iterator i=nodes.begin();
   node_t *aux = (i==nodes.end())? NULL : (*i);
   if (aux) {  // seznam neni prazdny
      while(++i!=nodes.end())
         if ((*i)->getvalue() < aux->getvalue()) aux = (*i);
      nodes.erase(aux);
   }
   return aux;
}

//////////////////////////////////////////////////////////////////////////////
// path_t

void path_t::clear() { ngraph_t::clear(); }

bool path_t::erase(node_t* Node, bool Edges) {
   return ngraph_t::erase(Node, Edges);
}

void path_t::erase(edge_t* Edge) {  ngraph_t::erase(Edge); }


node_t *path_t::insert_node() {             // omezuje pouze na neohodnocene uzly
   return ngraph_t::insert_node(0);
}

edge_t *path_t::insert_edge(node_t *From, node_t *To, int Value) {
   return (From!=To && Value>0)?
          ngraph_t::insert_edge(From, To, true, Value): NULL;
}

elemlst_t path_t::min_cost(node_t *From, node_t *To) {  // Boruvkova cesta :)
   elemlst_t outlst;
   if (nodes.find(From)!=nodes.end() && nodes.find(To)!=nodes.end()) {
      edgelst_t elst;
      node_t *node, *neighbour;
      nodelst_t open, close;   // nezpracovane uzly, minimalne ohodnocene

      // pocatecni konfigurace
      open.insert(From);  From->setvalue(0);

      // hleda cestu dokud je neco v open, tj. existuje cesta,
      // eventualne skonci driv pokud ma cil nejnizsi ohodnoceni,
      // tj. nejblize je to primo do koncoveho

      while((node=min_value_remove(open))!=NULL) {  // dokud nevyprazdni open
         close.insert(node);                // prida minimalne ohodnoceny
         if(node==To) {                     // min. cesta nalezena driv
            open.clear();
            elst.clear();                   // neni potreba hledat z cile dal
         }
         else elst = node->getedges();      // ziska seznam (neor.) hran

         // pokousi se snizit ohodnoceni vsem sousednim uzlum (v open)
         // pokud se do nich lze dostat (pres hranu) a ziskat nizsi cenu;
         // neni-li sousedni uzel jsete v seznmu, tak ho ohodnoti
         // (hodnota uzlu, ze ktereho prisel + hodnota hrany) a prida;
         // uzly ve kterych doslo ke snizeni ohodnoceni a nove uzly,
         // zapise do close nebot ty je poteba (znovu) zpracovat

         for(edgelst_t::iterator i=elst.begin(); i!=elst.end(); ++i) {
            neighbour=(*i)->isfrom(node);     // ziska souseda
            if(close.find(neighbour)==close.end()) {  // neni minimalni
               // ohodnoti nove a drazsim snizi ohodnoceni
               if (open.find(neighbour)==open.end() ||
                   node->getvalue()+(*i)->getvalue() < neighbour->getvalue()) {
                  neighbour->setvalue(node->getvalue() + (*i)->getvalue());
                  open.insert(neighbour);  // prida podud tam jeste neni
               }
            }
         }
      }
      // zpetny pruchod od ciloveho uzlu do pocatecniho; cesta vede pres
      // sousedni uzly pukud jejich cena + hodnota hrany, pres kterou jdeme
      // je rovna cene uzlu, ze ktereho prichazime; na vystup (outlst) se
      // zapise mnozina elementu gragu (hrany a uzly), kterymi vede cesta;
      // v zapsanych uzlech bude v hodnote uvedena jejich cena

      if (close.find(To)!=close.end()) {  // existuje cesta z From do To
         close.clear();   // od tedka jako zasobnik, pro nalezeni vsech cest
         close.insert(To);
         while (!close.empty()) {
            close.erase(node=(*close.begin()));  // odstrani dalsi reseni
            outlst.insert(node);                 // zapise uzel na vystup
            elst = node->getedges();             // ziska hrany uzlu
            for(edgelst_t::iterator i=elst.begin(); i!=elst.end(); ++i) {
               neighbour=(*i)->isfrom(node);     // ziska souseda
               if (node->getvalue()==neighbour->getvalue()+(*i)->getvalue()) {
                  if(outlst.find(neighbour)==outlst.end())
                  // pokud jese neni soucesti vysledku (spojeni cest)
                     close.insert(neighbour);    // zaznamena reseni
                  outlst.insert((*i));           // zapise hranu
               }
            }
         }
      }
   }
   return outlst;
}

//////////////////////////////////////////////////////////////////////////////
//  testovaci programek na cestu
#if DEBUG
int main() {                    // kontrolni programek (na dedicnosti, ...)
   path_t  p;

   node_t *s = p.insert_node();  cerr<<"S id:\t"<<s<<"\n";
   node_t *a = p.insert_node();  cerr<<"A id:\t"<<a<<"\n";
   node_t *b = p.insert_node();  cerr<<"B id:\t"<<b<<"\n";
   node_t *c = p.insert_node();  cerr<<"C id:\t"<<c<<"\n";
   node_t *g = p.insert_node();  cerr<<"G id:\t"<<g<<"\n\n";

   edge_t *sa = p.insert_edge(s, a, 10); cerr<<"SA id:\t"<<sa<<"\n";
   edge_t *sb = p.insert_edge(s, b, 6);  cerr<<"SB id:\t"<<sb<<"\n";
   edge_t *sc = p.insert_edge(s, c, 4);  cerr<<"SC id:\t"<<sc<<"\n";

   edge_t *cb = p.insert_edge(c, b, 1);  cerr<<"CB id:\t"<<cb<<"\n";
   edge_t *cg = p.insert_edge(c, g, 7);  cerr<<"CG id:\t"<<cg<<"\n";
   if(p.insert_edge(c, s, 4)) cerr<< "Chyba4\n";  // nesmi vlozit opacnou

   edge_t *ab = p.insert_edge(a, b, 2);  cerr<<"AB id:\t"<<ab<<"\n";
   edge_t *ag = p.insert_edge(a, g, 3);  cerr<<"AG id:\t"<<ag<<"\n";

   edge_t *bg = p.insert_edge(b, g, 9);  cerr<<"BG id:\t"<<bg<<"\n";
cerr<<"0\n";
   elemlst_t lst = p.min_cost(s,g);
cerr<<"1\n";
   for(elemlst_t::iterator i=lst.begin();i!=lst.end(); ++i)
     cerr << "id: " << (*i) <<"\tvalue: " << (*i)->getvalue() << "\n";
}

#endif
